import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:nexi_payment/nexi_payment.dart';

import 'models/api_front_office_qp_request.dart';

//typedef void CardFormViewCreatedCallback(CardFormViewController controller);
//typedef void OnPayClick(CardFormViewController controller);


class CardFormView extends StatefulWidget {

  final NexiPayment nexiPayment;

  CardFormView({
    @required this.nexiPayment,
    Key key,
  }) : super(key: key);



  @override
  State<StatefulWidget> createState() => _CardFormViewState();
}

class _CardFormViewState extends State<CardFormView> {


  @override
  Widget build(BuildContext context) {
    if (defaultTargetPlatform == TargetPlatform.android) {

      return GestureDetector(
        child:
        AndroidView(
        viewType: 'nexi_payment/cardform',
          gestureRecognizers: <Factory<OneSequenceGestureRecognizer>>[
            new Factory<OneSequenceGestureRecognizer>(
                  () => new EagerGestureRecognizer(),
            ),
          ].toSet(),
        onPlatformViewCreated: _onPlatformViewCreated,
      ),
        onTap: () => {
          print("focus requested"),
          FocusScope.of(context).requestFocus(FocusNode()
          )},
      );
    }
    return Text(
        '$defaultTargetPlatform is not yet supported by the nexi plugin');
  }

  void _onPlatformViewCreated(int id) {
    widget.nexiPayment.setCardViewId(id);
  }


}

class CardFormViewController {
  final String secretKey;
  final String environment;
  MethodChannel _channel;

  CardFormViewController(int id, this.secretKey, this.environment){
    _channel = new MethodChannel('nexi_payment/cardform_$id');
  }

  Future<bool>  pagaNonce(String alias, String codTrans, String currency, int amount) async {
    var res = await _channel.invokeMethod("pagaNonce", {
                                  "secretKey":this.secretKey,
                                  "alias": alias,
                                  "codTrans": codTrans,
                                  "currency": currency,
                                  "amount": amount,
                                  "environment": this.environment
                                });
    return res;
  }

}